#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <oci.h>


/* Include OCI-specific headers. */
#include <oratypes.h>
#include <ocidfn.h>
#ifdef __STDC__
#include <ociapr.h>
#else
#include <ocikpr.h>
#endif
#include <ocidem.h>

#include "ora.h"

#define DBG 1
#define PARSE_NO_DEFER           0
#define  DEFER_PARSE        1
#define  NATIVE             1
#define  VERSION_7          2
#define  MAX_ITEM_BUFFER_SIZE          33
#define  MAX_SELECT_LIST_SIZE          12


/* Declare structures for query information. */
struct describe
{
    sb4             dbsize;
    sb2             dbtype;
    sb1             buf[MAX_ITEM_BUFFER_SIZE];
    sb4             buflen;
    sb4             dsize;
    sb2             precision;
    sb2             scale;
    sb2             nullok;
};

struct define
{
    ub1             buf[MAX_ITEM_BUFFER_SIZE];
    float           flt_buf;
    sword           int_buf;
    sb2             indp;
    ub2             col_retlen, col_retcode;
};

static Lda_Def lda;
static Cda_Def cda;
static ub4     hda[HDA_SIZE/(sizeof(ub4))];
static sword numwidth = 8;
#ifdef DBG
FILE *fp;
#endif

/* Define arrays of describe and define structs. */
static struct describe desc[MAX_SELECT_LIST_SIZE];
static struct define   def[MAX_SELECT_LIST_SIZE];

FILE *fp;

/* Prototipos de funciones locales */
int fetch_row(Cda_Def *,sword , char **);
sword describe_define(Cda_Def *cda);

void oralogoff (void)
{
          ologof(&cda);
          ologof(&lda);
}

int oralogin(char *user, char *passwd, char *dbname,char *errbuf) 
{

    if (olog(&lda, (ub1 *)hda, user, -1, passwd, -1,
                 (text *) dbname, -1, (ub4)OCI_LM_DEF)) {
       oerhms(&lda, ((Cda_Def *)&lda)->rc, errbuf, (sword) 512);
      return(-1);
  }

return (0);
}

int oraexec(char *comando,char *errbuf)
{ 
int rc=0;

 if (oopen(&cda, &lda, (text *) 0, -1, -1, (text *) 0, -1))      
  {
    rc=((&cda)->rc);
    oerhms(&lda, rc, errbuf, (sword) 512);
    oralogoff();
    return (rc);
  }
                                                                                
  if (oparse(&cda, comando, (sb4) -1, DEFER_PARSE,                
               (ub4) VERSION_7))
  {
    rc=((&cda)->rc);
    oerhms(&lda, rc, errbuf, (sword) 512);
    oralogoff();
    return (rc);
  }

  if (oexec(&cda))
  {
    rc=((&cda)->rc);
    oerhms(&lda, rc, errbuf, (sword) 512);
    oralogoff();
    return (rc);
  }

return rc;
}


int oracursor(int ncols,char *errbuf, char **fila)
{ 

if (fetch_row(&cda,ncols,fila)) {
    oralogoff();
return 0;
} else {
return 1;
}

}

int oraopencur(char *comando,char *errbuf, int *ncol)
{ 
int rc=0;
sword ncols=0;

#ifdef DBG
fp=fopen("/tmp/oralink_ora.log","w");
#endif

 if (oopen(&cda, &lda, (text *) 0, -1, -1, (text *) 0, -1))        
  {
    rc=((&cda)->rc);
    oerhms(&lda, rc, errbuf, (sword) 512);
    oralogoff();
    return (rc);
  }
                                                                                
  if (oparse(&cda, comando, (sb4) -1, PARSE_NO_DEFER,               
               (ub4) VERSION_7))
  {
    rc=((&cda)->rc);
    oerhms(&lda, rc, errbuf, (sword) 512);
    oralogoff();
    return (rc);
  }

  ncols=describe_define(&cda);
  if (ncols<0)
  {
    rc=((&cda)->rc);
    oerhms(&lda, rc, errbuf, (sword) 512);
    oralogoff();
    return (rc);
  }

  if (oexec(&cda))
  {
    rc=((&cda)->rc);
    oerhms(&lda, rc, errbuf, (sword) 512);
    oralogoff();
    return (rc);
  }
 
*ncol=ncols; 
return 0;
}

/*  Describe select-list items. */

sword describe_define(Cda_Def *cda)
{
    sword col, deflen, deftyp;
    static ub1 *defptr;

    /* Describe the select-list items. */
    for (col = 0; col < MAX_SELECT_LIST_SIZE; col++)
    {
        desc[col].buflen = MAX_ITEM_BUFFER_SIZE;
        if (odescr(cda, col + 1, &desc[col].dbsize,
                   &desc[col].dbtype, &desc[col].buf[0],
                   &desc[col].buflen, &desc[col].dsize,
                   &desc[col].precision, &desc[col].scale,
                   &desc[col].nullok))
        {
            /* Break on end of select list. */
            if (cda->rc == VAR_NOT_IN_LIST)
                break;
            else
            {
                return -1;
            }
        }
        /* adjust sizes and types for display */
        switch (desc[col].dbtype)
        {
        case NUMBER_TYPE:
            desc[col].dbsize = numwidth;
            /* Handle NUMBER with scale as float. */
            if (desc[col].scale != 0)
            {
                defptr = (ub1 *) &def[col].flt_buf;
                deflen = (sword) sizeof(float);
                deftyp = FLOAT_TYPE;
                desc[col].dbtype = FLOAT_TYPE;
            }
            else
            {
                defptr = (ub1 *) &def[col].int_buf;
                deflen = (sword) sizeof(sword);
                deftyp = INT_TYPE;
                desc[col].dbtype = INT_TYPE;
            }
            break;
        default:
            if (desc[col].dbtype == DATE_TYPE)
                desc[col].dbsize = 9;
            if (desc[col].dbtype == ROWID_TYPE)
                desc[col].dbsize = 18;
            defptr = def[col].buf;
            deflen = desc[col].dbsize > MAX_ITEM_BUFFER_SIZE ?
              MAX_ITEM_BUFFER_SIZE : desc[col].dbsize + 1;
            deftyp = STRING_TYPE;
            break;
        }
        if (odefin(cda, col + 1,
                   defptr, deflen, deftyp,
                   -1, &def[col].indp, (text *) 0, -1, -1,
                   &def[col].col_retlen,
                   &def[col].col_retcode))
        {
            return -1;
        }
    }
    return col;
}

void alloc_row(int ncols, char **fila)
{
    sword col;

        for (col = 0; col < ncols ; col++)
        {
           switch (desc[col].dbtype)
                {
                case FLOAT_TYPE:
                    fila[col]= (char *)malloc(numwidth+40);
                    break;
                case INT_TYPE:
                    fila[col]= (char *)malloc(numwidth+10);
                    break;
                default:
                    fila[col]=malloc(desc[col].dbsize+100);
                }
        }
}

int fetch_row(Cda_Def *cda,sword ncols, char **fila)
{
    sword col;


        if (ofetch(cda))
        {
            if (cda->rc == NO_DATA_FOUND) {
#ifdef DBG
fprintf(fp,"fetch : NO_DATA_FOUND..\n"); fflush(fp);
#endif
                return -1;
            }
            if (cda->rc != NULL_VALUE_RETURNED) {
#ifdef DBG
fprintf(fp,"fetch : NULL_RET.. rc : %d\n",cda->rc); fflush(fp);
#endif
                return -2;
            }
        }
#ifdef DBG
fprintf(fp,"ncols= %d\n",ncols); fflush(fp);
#endif
        for (col = 0; col < ncols ; col++)
        {
            /* Check col. return code for null.  If
               null, print n spaces, else print value. */
            if (def[col].indp < 0) {
#ifdef DBG
fprintf(fp,"fetch : NULL COL : %d\n",col); fflush(fp);
#endif
                sprintf(fila[col],"");
            }
            else
            {
                switch (desc[col].dbtype)
                {
                case FLOAT_TYPE:
#ifdef DBG
fprintf(fp,"fetch : FLOAT COL : %d\n",col); fflush(fp);
fprintf(fp,"fetch : VAL : %.2f\n",def[col].flt_buf); fflush(fp);
#endif
                    sprintf(fila[col],"%.2f", def[col].flt_buf);
                    break;
                case INT_TYPE:
#ifdef DBG
fprintf(fp,"fetch : INT COL : %d\n",col); fflush(fp);
fprintf(fp,"fetch : VAL : %d\n",def[col].int_buf); fflush(fp);
#endif
                    sprintf(fila[col],"%d",def[col].int_buf);
                    break;
                default:
#ifdef DBG
fprintf(fp,"fetch : OTR COL : %d\n",col); fflush(fp);
fprintf(fp,"fetch : VAL : %s\n",def[col].buf); fflush(fp);
#endif
                    sprintf(fila[col],"%s", def[col].buf);
                }
            }
        }
return 0;
}

