#include "postgres.h"

#include "libpq-fe.h"

#include "fmgr.h"
#include "funcapi.h"
#include "access/tupdesc.h"
#include "access/heapam.h"
#include "catalog/catname.h"
#include "catalog/namespace.h"
#include "catalog/pg_index.h"
#include "catalog/pg_type.h"
#include "executor/executor.h"
#include "executor/spi.h"
#include "lib/stringinfo.h"
#include "nodes/nodes.h"
#include "nodes/execnodes.h"
#include "nodes/pg_list.h"
#include "parser/parse_type.h"
#include "tcop/tcopprot.h"
#include "utils/builtins.h"
#include "utils/fmgroids.h"
#include "utils/array.h"
#include "utils/lsyscache.h"
#include "utils/syscache.h"

#include "oralink.h"
#include "ora.h"

/* Global */
List	   *res_id = NIL;
int			res_id_index = 0;
char       **row;
int ncols,rc;


#define GET_STR(textp) DatumGetCString(DirectFunctionCall1(textout, PointerGetDatum(textp)))
#define OLPREFIX "ol"

/*
 * Ejecuta un SQL contra Oracle
 */
PG_FUNCTION_INFO_V1(oralink_exec);
Datum
oralink_exec(PG_FUNCTION_ARGS)
{
	char	   *user = GET_STR(PG_GETARG_TEXT_P(0));
	char	   *password = GET_STR(PG_GETARG_TEXT_P(1));
	char	   *tns = GET_STR(PG_GETARG_TEXT_P(2));
	char	   *sql = GET_STR(PG_GETARG_TEXT_P(3));
	text	   *result_text;
        int rc;
        char       error[1024];

  rc=oralogin(user,password,tns,error);
  if (rc) {
    elog(ERROR, "oralink connect: %s", error);
    result_text = DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(error)));
    PG_RETURN_TEXT_P(result_text);
  }


  if (oraexec(sql,error)) {
    elog(ERROR, "oralink exec: %s", error);
    result_text = DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(error)));
    PG_RETURN_TEXT_P(result_text);
  }

  oralogoff();

 result_text = DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum("OK")));
 PG_RETURN_TEXT_P(result_text);
}

/*
 * Ejecuta un cursor contra Oracle  
 */

PG_FUNCTION_INFO_V1(oralink_cursor);
Datum
oralink_cursor(PG_FUNCTION_ARGS)
{

	char	   *user = GET_STR(PG_GETARG_TEXT_P(0));
	char	   *password = GET_STR(PG_GETARG_TEXT_P(1));
	char	   *tns = GET_STR(PG_GETARG_TEXT_P(2));
	char	   *sql = GET_STR(PG_GETARG_TEXT_P(3));
	FuncCallContext *funcctx;
	TupleDesc	tupdesc = NULL;
	int			call_cntr;
        int fi;
	int			max_calls;
	TupleTableSlot *slot;
	AttInMetadata *attinmeta;
	MemoryContext oldcontext;
        char       error[512];
	text	   *result_text;
        char tipo[80];
#ifdef DBG
       FILE *fp;
        int f; 
#endif

	/* stuff done only on the first call of the function */
	if (SRF_IS_FIRSTCALL())
	{
#ifdef DBG
        fp=fopen("/tmp/oralink.log","w");
        fprintf (fp,"\nStart:\n");
#endif


        rc=oralogin(user,password,tns,error);
        if (rc) {
            elog(ERROR, "oralink connect: %s", error);
             result_text = DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(error)));
            PG_RETURN_TEXT_P(result_text);
        }

#ifdef DBG
        fprintf (fp,"login\n");
        fflush (fp);
#endif
         rc=oraopencur(sql,error, &ncols);
        if (rc) {
            elog(ERROR, "oralink opencursor: %s", error);
             result_text = DatumGetTextP(DirectFunctionCall1(textin, CStringGetDatum(error)));
            PG_RETURN_TEXT_P(result_text);
        }

#ifdef DBG
        fprintf (fp,"opencur\n");
        fflush (fp);
#endif
        row= (char **) malloc (ncols*sizeof (char *));

         alloc_row(ncols,row);

		funcctx = SRF_FIRSTCALL_INIT();

		oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);

               /* funcctx->max_calls = PG_GETARG_UINT32(5); */
                funcctx->max_calls = 1000000;

        /*sprintf(tipo,"oralink.%s%d",OLPREFIX,ncols); */
        tupdesc = CreateTemplateTupleDesc(ncols, false);
        for (fi=1;fi<=ncols;fi++)
        TupleDescInitEntry(tupdesc, (AttrNumber) fi, "position",
  			   TEXTOID, -1, 0);
        
#ifdef DBG
        fprintf (fp,"\n\nTupdesc:\n");
        fprintf (fp,"tupdesc->natts : %d\n",tupdesc->natts);
        fprintf (fp,"tupdesc->attrs :\n");
        for (f=0;f<tupdesc->natts;f++) {
        fprintf (fp,"attrs[%d] : %s\n",f,(tupdesc->attrs)[f]);
        fprintf (fp,"attrs[%d] : %p\n",f,(tupdesc->attrs)[f]);
        fprintf (fp,"attrs[%d] : %d\n",f,(tupdesc->attrs)[f]);
        }
        fprintf (fp,"tupdesc->constr : %p\n",tupdesc->constr);
        fprintf (fp,"tupdesc->hasoid : %d\n",tupdesc->tdhasoid);
        fflush(fp); 
#endif
        
        slot = TupleDescGetSlot(tupdesc);

        funcctx->slot = slot;

        /*
         * Generate attribute metadata needed later to produce tuples from raw
         * C strings
         */
        attinmeta = TupleDescGetAttInMetadata(tupdesc);
        funcctx->attinmeta = attinmeta;

        MemoryContextSwitchTo(oldcontext);
	}

	/* stuff done on every call of the function */
	funcctx = SRF_PERCALL_SETUP();

	call_cntr = funcctx->call_cntr;
	max_calls = funcctx->max_calls;
	slot = funcctx->slot;
	attinmeta = funcctx->attinmeta;

#ifdef DBG
        fprintf (fp,"Mainncols : %d\n",ncols);
        fprintf (fp,"max_calls : %d\n",max_calls);
        fprintf (fp,"call_cntr : %d\n",call_cntr);
        fflush (fp);
#endif
	rc=oracursor(ncols,error, row);
	/*if (call_cntr < max_calls) */
	 if (rc)
	{
		HeapTuple	tuple;
		Datum		result;

		/* build the tuple */
		tuple = BuildTupleFromCStrings(attinmeta, row);

		/* make the tuple into a datum */
		result = TupleGetDatum(slot, tuple);

		SRF_RETURN_NEXT(funcctx, result);
	}
	else
/* do when there is no more left */
	{
		SRF_RETURN_DONE(funcctx);
	}
}

